/*  Floating Point(Double) Calc Subroutines
        Copyright(c) 2007,  S.SAKAI     2007.3.5    */
#include    "fltmpu.h"

int     _dnormb(unsigned char *);

/*      s = 0(x >= 0)   x[7]..x[0] = 0xff....ff;
        s != 0(x < 0)   x[7]..x[0] = 0x7f....ff;    */
void _ovrfld(unsigned char *x,int s)
{ unsigned long *ux;

    ux = (unsigned long *)x;
#if     FLTMPU
    *(ux+1) = 0xffffffff;
    if (s == 0) *ux = 0x7fffffff;
    else *ux = 0xffffffff;
#else
    *ux = 0xffffffff;
    if (s == 0) *(ux+1) = 0x7fffffff;
    else *(ux+1) = 0xffffffff;
#endif
}

/*      x[7]..x[0] = 0x00..00;  */
void _undfld(unsigned char *x)
{ unsigned long *ux;

    ux = (unsigned long *)x;
    *ux = 0x0;
    *(ux+1) = 0x0;
}

/*      x[7]..x[0] += 1;    */
void _regdp1(unsigned char *x)
{ unsigned long     *ux;

    ux = (unsigned long *)x;
#if     FLTMPU
    *(ux+1) += 1;
    if (*(ux+1) == 0) *ux += 1;
#else
    *ux += 1;
    if (*ux == 0) *(ux+1) += 1;
#endif
}

/*      x[7]..x[0] <<= 1;   */
void _regshld(unsigned char *x)
{ unsigned long *ux;

    ux = (unsigned long *)x;
#if     FLTMPU
    *ux <<= 1;
    if (x[4] & 0x80) x[3] |= 0x1;
    *(ux+1) <<= 1;
#else
    *(ux+1) <<= 1;
    if (x[3] & 0x80) x[4] |= 0x1;
    *ux <<= 1;
#endif
}

/*      x[7]..x[0] >>= 1;   */
void _regshrd(unsigned char *x)
{ unsigned long *ux;

    ux = (unsigned long *)x;
#if     FLTMPU
    *(ux+1) >>= 1;
    if (x[3] & 0x1) x[4] |= 0x80;
    *ux >>= 1;
#else
    *ux >>= 1;
    if (x[4] & 0x1) x[3] |= 0x80;
    *(ux+1) >>= 1;
#endif
}

/*      x[7]..x[0] *= 10;   */
void _mlt10d(unsigned char *x)
{ int               i;
  unsigned long     a,b;
  unsigned short    *ux;


    b = 0;
#if     FLTMPU
    for (i=3;i>=0;i--) {
#else
    for (i=0;i<4;i++) {
#endif
        ux = (unsigned short *)(x+i*2);
        a = *ux;
        a *= 10;
        a += b;
        *ux = a;
        b = a >> 16;
    }
}

/*      x[7]..x[0] /= 10;   */
void _div10d(unsigned char *x)
{ int               i;
  unsigned long     a,b;
  unsigned short    *ux;

    ux = (unsigned short *)x;
    b = 0;
#if     FLTMPU
    for (i=0;i<4;i++) {
#else
    for (i=3;i>=0;i--) {
#endif
        a = *(ux+i);
        a += (b << 16);
        *(ux+i) = a / 10;
        b = a % 10;
    }
    if (b > 4) _regdp1(x);
}

/*      x[7]..x[0] == 0     ret = 1;
        x[7]..x[0] != 0     ret = 0;    */
int _zrchkd(unsigned char *x)
{ unsigned long     *ux;

    ux = (unsigned long *)x;
    if (*ux) return(0);
    if (*(ux+1)) return(0);
    return(1);
}

void _dneg(unsigned char *x)
{ unsigned long     *ux;

    ux = (unsigned long *)x;
    *ux = ~(*ux);
    *(ux+1) = ~(*(ux+1));
    _regdp1(x);

}

/*      Double -> w &    */
int _dnorm(unsigned char *x)
{ unsigned long     a,b;
  unsigned long     *ux;

    ux = (unsigned long *)x;
#if     FLTMPU
    a = *ux;
#else
    a = *(ux+1);
#endif
    a &= 0xfff00000;
    a >>= 20;

#if     FLTMPU
    *ux &= 0xfffff;
    *ux |= 0x100000;
    *ux <<= 8;
    b = *(ux+1);
    *(ux+1) <<= 8;
#else
    *(ux+1) &= 0xfffff;
    *(ux+1) |= 0x100000;
    *(ux+1) <<= 8;
    b = *ux;
    *ux <<= 8;
#endif
    b >>= 24;
#if     FLTMPU
    *ux |= b;
#else
    *(ux+1) |= b;
#endif
    return(a);
}

/*      w &  -> Double   */
void _danorm(unsigned char *x,int z)
{ unsigned long     a,b;
  unsigned long     *ux;

    ux = (unsigned long *)x;
#if     FLTMPU
    a = *(ux+1);
    *(ux+1) >>= 8;
    b = *ux;
    *ux >>= 8;
    *ux &= 0x1fffff;
#else
    a = *ux;
    *ux >>= 8;
    b = *(ux+1);
    *(ux+1) >>= 8;
    *(ux+1) &= 0x1fffff;
#endif

    b <<= 24;
#if     FLTMPU
    *(ux+1) |= b;
#else
    *ux |= b;
#endif
    if (a & 0x80) _regdp1(x);

#if     FLTMPU
    if (x[1] & 0x20) {
#else
    if (x[6] & 0x20) {
#endif
        z++;
        _regshrd(x);
    }
#if     FLTMPU
    x[1] &= 0xf;
#else
    x[6] &= 0xf;
#endif

    z <<= 20;
#if     FLTMPU
    *ux |= z;
#else
    *(ux+1) |= z;
#endif
}

/*      x[7]..x[0] >>= n;   */
void _dnormshr(unsigned char *x,int n)
{ unsigned long     *ux,a,b,c,c2;

    if (n == 0) return;

    ux = (unsigned long *)x;
    if (n > 32) a = n - 32;
    else a = n;
    b = 1;
    b <<= (a - 1);

    if (n >= 64) {
#if     FLTMPU
#else
        if ((n == 64) && (x[7] & 0x80)) *ux = 1;
        else *ux = 0;
        *(ux+1) = 0;
#endif
    }
    else if (n > 32) {
#if     FLTMPU
        c = *ux;
        *(ux+1) = *ux >> a;
#else
        c = *(ux+1);
        *ux = *(ux+1) >> a;
#endif
        if (c & b) {
#if     FLTMPU
            *(ux+1) += 1;
            if (*(ux+1) == 0) *ux = 1;
            else *ux = 0;
        }
        else *ux = 0;
#else
            *ux += 1;
            if (*ux == 0) *(ux+1) = 1;
            else *(ux+1) = 0;
        }
        else *(ux+1) = 0;
#endif
    }
    else if (n == 32) {
#if     FLTMPU
        c = x[4];
        *(ux+1) = *ux;
        *ux = 0;
        if (c & 0x80) _regdp1(x);
#else
        c = x[3];
        *ux = *(ux+1);
        *(ux+1) = 0;
        if (c & 0x80) _regdp1(x);
#endif
    }
    else {
#if     FLTMPU
        c = *(ux+1);
        *(ux+1) >>= a;
        c2 = *ux;
        *ux >>= a;
#else
        c = *ux;
        *ux >>= a;
        c2 = *(ux+1);
        *(ux+1) >>= a;
#endif
        c2 <<= (32 - a);
#if     FLTMPU
        *(ux+1) |= c2;
#else
        *ux |= c2;
#endif
        if (c & b) _regdp1(x);
    }
}

/*      x[7]..x[0] <<= n;   */
void _dnormshl(unsigned char *x,int n)
{ int               m;
  unsigned long     *ux,a,b;

    if (n == 0) return;

    ux = (unsigned long *)x;
    if (n > 32) a = n - 32;
    else a = n;

    if (n >= 64) {
        *ux = 0;
        *(ux+1) = 0;
    }
    else if (n > 32) {
#if     FLTMPU
        *ux = *(ux+1) << a;
        *(ux+1) = 0;
#else
        *(ux+1) = *ux << a;
        *ux = 0;
#endif
    }
    else if (n == 32) {
#if     FLTMPU
        *ux = *(ux+1);
        *(ux+1) = 0;
#else
        *(ux+1) = *ux;
        *ux = 0;
#endif
    }
    else {
#if     FLTMPU
        *ux <<= a;
        b = *(ux+1);
        *(ux+1) <<= a;
#else
        *(ux+1) <<= a;
        b = *ux;
        *ux <<= a;
#endif
        b >>= (32 - a);
#if     FLTMPU
        *ux |= b;
#else
        *(ux+1) |= b;
#endif
    }
}

/*      x[7]..x[0] >  y[7]..y[0]        ret = 1;
        x[7]..x[0] == y[7]..y[0]        ret = 0;
        x[7]..x[0] <  y[7]..y[0]        ret = -1;   */
int _dcmpxy(unsigned char *x,unsigned char *y)
{ unsigned long     *ux,*uy;

    ux = (unsigned long *)x;
    uy = (unsigned long *)y;

#if     FLTMPU
    if (*ux > *uy) return(1);
    if (*ux < *uy) return(-1);
    if (*(ux+1) > *(uy+1)) return(1);
    if (*(ux+1) < *(uy+1)) return(-1);
#else
    if (*(ux+1) > *(uy+1)) return(1);
    if (*(ux+1) < *(uy+1)) return(-1);
    if (*ux > *uy) return(1);
    if (*ux < *uy) return(-1);
#endif
    return(0);
}

/*      x[7]..x[0] += z;    */
void _dadd1b(unsigned char *x,int z)
{ int               i;
  unsigned long     a,b;
  unsigned short    *ux;

    b = z;
    ux = (unsigned short *)x;
#if     FLTMPU
    for (i=3;i>=0;i--) {
#else
    for (i=0;i<4;i++) {
#endif
        a = *(ux+i);
        a += b;
        *(ux+i) = a;
        b = (a >> 16);
        if (b == 0) break;
    }
}

int _dnorma(unsigned char *x)
{ int           a,b,f;
  unsigned long *ux;

    ux = (unsigned long *)x;

    b = 0;
    f = 0;
    while(1) {
#if     FLTMPU
        a = x[0] & 0xf0;
#else
        a = x[7] & 0xf0;
#endif
        if (a == 0x10) break;
        if (a < 0x10) {
            f = 0;
            _regshld(x);
            b -= 1;
        }
        else {
            if (*ux & 0x1) f = 1;
            else f = 0;
            _regshrd(x);
            b += 1;
        }
    }
    if (f) {
        _regdp1(x);
#if     FLTMPU
        a = x[0] & 0xf0;
#else
        a = x[7] & 0xf0;
#endif
        if (a != 0x10) {
            _regshrd(x);
            b += 1;
        }
    }
    return(b);
}

int _dnormb(unsigned char *x)
{ int           b,f;

    b = 0;
    f = 0;
    while(1) {
#if     FLTMPU
        if (x[0] == 0x1) break;
        if (x[0] == 0) {
#else
        if (x[7] == 0x1) break;
        if (x[7] == 0) {
#endif
            f = 0;
            _regshld(x);
            b -= 1;
        }
        else {
#if     FLTMPU
            f = x[7] & 0x1;
#else
            f = x[0] & 0x1;
#endif
            _regshrd(x);
            b += 1;
        }
    }
    if (f) {
        _regdp1(x);
#if     FLTMPU
        if (x[0] != 0x1) {
#else
        if (x[7] != 0x1) {
#endif
            _regshrd(x);
            b += 1;
        }
    }
    return(b);
}
