/*      SCI Initialize & Subroutins for SH7144
    Copyright(c) 2007, S.SAKAI  2007.6.14       */

#include    "iodefine.h"
#include    "float.h"

unsigned char   rx1_buff[256];
unsigned char   tx1_buff[256];
unsigned char   rx1_rd,rx1_wr;
unsigned char   tx1_rd,tx1_wr;

// Serial Port0 Initialize
void sci1_init(void)
{ unsigned char dummy;

    rx1_rd = 0;
    rx1_wr = 0;
    tx1_rd = 0;
    tx1_wr = 0;

	MST.CR1.WORD &= ~0x0002;			// SCI1 Standby off

    INTC.IPRF.WORD &= ~0x000f;
    INTC.IPRF.WORD |= 0x0007;         	// SCI1 InterruptLevel = 7

    SCI1.SDCR.BYTE = 0xf2;          	// LSB first,Normal

    SCI1.SCR.BYTE = 0x00;           	// Disable TxInt
                                        // Disable RxInt
                                        // Disable TX,RX
    SCI1.SMR.BYTE = SMRSCI1;
    SCI1.BRR  = ((CLK2 * 1000 / 32 + BPSSCI1 / 2) / BPSSCI1) - 1;
    dummy = SCI1.SSR.BYTE;          	// Clear Status
    SCI1.SSR.BYTE = 0x04;

    SCI1.SCR.BYTE |= 0x70;          	// Enable RxInt,Tx,Rx
}

void sci1_putc(unsigned char data)
{
    tx1_buff[tx1_wr++] = data;
    SCI1.SCR.BYTE |= 0x80;          	// Enable TxInt
}

short sci1_getc(void)
{ short data;

    data = -1;
    if (rx1_rd != rx1_wr) {
        data = rx1_buff[rx1_rd++];
    }
    return(data);
}

// Rx Error Check
#pragma interrupt (int_eri1)
void int_eri1(void)
{
    SCI1.SSR.BYTE &= 0x80;          	// Error flag Clear
}

// Rx Int
#pragma interrupt (int_rxi1)
void int_rxi1(void)
{ unsigned char data;

    data = SCI1.RDR;                	// Rec
    SCI1.SSR.BYTE &= ~0x40;         	// Clear RDRF

    if (rx1_wr+1 != rx1_rd) {
        rx1_buff[rx1_wr++] = data;
    }
}

// Tx Int
#pragma interrupt (int_txi1)
void int_txi1(void)
{
    if (tx1_rd != tx1_wr) {
        SCI1.TDR = tx1_buff[tx1_rd++];  // Transmit 1 Byte
        SCI1.SSR.BYTE &= ~0x80;     	// Clear TDRE
    }
    else {
        SCI1.SCR.BYTE &= ~0x80;     	//  Disable TxInt
        SCI1.SCR.BYTE &= ~0x04;     	// Disable TxEndInt
    }
}

// TxEndInt
#pragma interrupt (int_tei1)
void int_tei1(void)
{
    SCI1.SCR.BYTE &= ~0x04;         	// Disable TxEndInt
}
