/*  RS-232C Program for Floating Point Calculation Library Test Program
        Copyright(c) 2007, S.SAKAI  2007.6.13
*/
/*
*******************************************************************************
**
**  This device driver was created by Applilet for the V850ES/JG2, V850ES/JJ2
**  32-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002-2006
**  All rights reserved by NEC Electronics Corporation
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :  serial.c
**  Abstract :  This file implements a device driver for the SERIAL module
**  APIlib :    V850ESJx2.lib V1.70 [7 Dec. 2006]
**
**  Device :    uPD70F3716
**
**  Compiler :  NEC/CA850
**
*******************************************************************************
*/

#include "macrodriver.h"
#include "serial.h"

#pragma interrupt INTUA0R MD_INTUA0R
#pragma interrupt INTUA0T MD_INTUA0T

unsigned char   rx0_buff[256];
unsigned char   tx0_buff[256];
unsigned char   rx0_rd,rx0_wr;
unsigned char   tx0_rd,tx0_wr;

char    _txf;

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**  This function initializes UARTA0.
**
**  Parameters:
**  None
**
**  Returns:
**  None
**
**-----------------------------------------------------------------------------
*/
void UART0_Init( void )
{
    ClrIORBit(UA0CTL0, 0x80);       /* stop uarta0 */
    ClrIORBit(UA0CTL0, 0x40);
    ClrIORBit(UA0CTL0, 0x20);

    SetIORBit(UA0TIC, 0x40);        /* stop interrupt */
    SetIORBit(UA0RIC, 0x40);
    ClrIORBit(UA0TIC, 0x80);
    ClrIORBit(UA0RIC, 0x80);
    ClrIORBit(PFC3, 0x03);          /* port setting */
    SetIORBit(PMC3, 0x03);
    SetIORBit(UA0CTL0, 0x10);       /* LSB */
    ClrIORBit(UA0CTL0, 0xc);        /* none parity */
    SetIORBit(UA0CTL0, 0x2);        /* 8 bits data frame */
    ClrIORBit(UA0CTL0, 0x1);        /* 1 bit stop bit */
    ClrIORBit(UA0OPT0, 0x2);        /* transfer data level: normal */
    ClrIORBit(UA0OPT0, 0x1);        /* receive data level: normal */
    UA0CTL1 = UART0_BAUDRATE_M0;        /* baudrate setting */
    UA0CTL2 = UART0_BAUDRATE_K0;
    SetIORBit(UA0RIC, 0x7);         /* receive end interrupt priority is lowest */
    SetIORBit(UA0TIC, 0x7);         /* transmit end interrupt priority is lowest */
    ClrIORBit(UA0RIC, 0x40);        /* enable reception interrupt servicing */
    ClrIORBit(UA0TIC, 0x40);        /* enable transmission interrupt servicing */

    SetIORBit(UA0CTL0, 0x80);
    SetIORBit(UA0CTL0, 0x60);

    _txf = 0;
    return;
}

/**------------------------------------------------------------------------------**/
void sci1_putc(unsigned char data)
{
    if ((UA0STR & 0x80) == 0) UA0TX = data;
    else {
        if (_txf) {
            UA0TX = data;
            _txf = 0;
        }
        else tx0_buff[tx0_wr++] = data;
    }
}

/**------------------------------------------------------------------------------**/
void sci1_puts(unsigned char *s,unsigned short n)
{
    while(n--) tx0_buff[tx0_wr++] = *s++;
    if ((UA0STR & 0x80) == 0) {
        UA0TX = tx0_buff[tx0_rd++];         /* Transmit 1 Byte */
    }
}

/**------------------------------------------------------------------------------**/
short sci1_getc(void)
{ short data;

    data = -1;
    if (rx0_rd != rx0_wr) {
        data = rx0_buff[rx0_rd++];
    }
    return(data);
}

/**------------------------------------------------------------------------------**/
__interrupt void MD_INTUA0T( void )
{
    if (tx0_rd != tx0_wr) {
        UA0TX = tx0_buff[tx0_rd++];     /* Transmit 1 Byte */
        _txf = 0;
    }
    else _txf = 1;
}

__interrupt void MD_INTUA0R( void )
{ unsigned char data;

    data = UA0RX;
    if (((UA0STR & 0x7) == 0) && (rx0_wr+1 != rx0_rd)) {
        rx0_buff[rx0_wr++] = data;
    }
}
